@@ -1,4 +1,4 @@
-/*	$OpenBSD: extern.h,v 1.156 2022/09/03 21:24:02 job Exp $ */
+/*	$OpenBSD: extern.h,v 1.162 2022/11/29 10:33:09 claudio Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -24,15 +24,6 @@
 #include <openssl/x509.h>
 #include <openssl/x509v3.h>
 
-/*
- * Distrusted hosts (loaded from skipfile).
- */
-struct skiplistentry {
-	LIST_ENTRY(skiplistentry)	 entry;
-	char				*value; /* FQDN */
-};
-LIST_HEAD(skiplist, skiplistentry);
-
 /*
  * Enumeration for ASN.1 explicit tags in RSC eContent
  */
@@ -183,6 +174,8 @@ enum rtype {
 	RTYPE_FILE,
 	RTYPE_RSC,
 	RTYPE_ASPA,
+	RTYPE_TAK,
+	RTYPE_GEOFEED,
 };
 
 enum location {
@@ -212,6 +205,7 @@ struct mft {
 	char		*seqnum; /* manifestNumber */
 	char		*aia; /* AIA */
 	char		*aki; /* AKI */
+	char		*sia; /* SIA signedObject */
 	char		*ski; /* SKI */
 	char		*crl; /* CRL file name */
 	unsigned char	 crlhash[SHA256_DIGEST_LENGTH];
@@ -247,6 +241,7 @@ struct roa {
 	int		 valid; /* validated resources */
 	char		*aia; /* AIA */
 	char		*aki; /* AKI */
+	char		*sia; /* SIA signedObject */
 	char		*ski; /* SKI */
 	time_t		 expires; /* do not use after */
 };
@@ -274,6 +269,55 @@ struct rsc {
 	time_t		 expires; /* Not After of the RSC EE */
 };
 
+/*
+ * Datastructure representing the TAKey sequence inside TAKs.
+ */
+struct takey {
+	char		**comments; /* Comments */
+	size_t		 commentsz; /* number of Comments */
+	char		**uris; /* CertificateURI */
+	size_t		 urisz; /* number of CertificateURIs */
+	unsigned char	*pubkey; /* DER encoded SubjectPublicKeyInfo */
+	size_t		 pubkeysz;
+	char		*ski; /* hex encoded SubjectKeyIdentifier of pubkey */
+};
+
+/*
+ * A Signed TAL (TAK) draft-ietf-sidrops-signed-tal-12
+ */
+struct tak {
+	int		 talid; /* TAK covered by what TAL */
+	struct takey	*current;
+	struct takey	*predecessor;
+	struct takey	*successor;
+	char		*aia; /* AIA */
+	char		*aki; /* AKI */
+	char		*sia; /* SIA signed Object */
+	char		*ski; /* SKI */
+	time_t		 expires; /* Not After of the TAK EE */
+};
+
+/*
+ * A single geofeed record
+ */
+struct geoip {
+	struct cert_ip	*ip;
+	char		*loc;
+};
+
+/*
+ * A geofeed file
+ */
+struct geofeed {
+	struct geoip	*geoips; /* Prefix + location entry in the CSV */
+	size_t		 geoipsz; /* number of IPs */
+	char		*aia; /* AIA */
+	char		*aki; /* AKI */
+	char		*ski; /* SKI */
+	time_t		 expires; /* Not After of the Geofeed EE */
+	int		 valid; /* all resources covered */
+};
+
 /*
  * A single Ghostbuster record
  */
@@ -281,6 +325,7 @@ struct gbr {
 	char		*vcard;
 	char		*aia; /* AIA */
 	char		*aki; /* AKI */
+	char		*sia; /* SIA signedObject */
 	char		*ski; /* SKI */
 };
 
@@ -297,11 +342,12 @@ struct aspa {
 	int			 talid; /* TAL the ASPA is chained up to */
 	char			*aia; /* AIA */
 	char			*aki; /* AKI */
+	char			*sia; /* SIA signedObject */
 	char			*ski; /* SKI */
-	uint32_t	 	 custasid; /* the customerASID */
+	uint32_t		 custasid; /* the customerASID */
 	struct aspa_provider	*providers; /* the providers */
 	size_t			 providersz; /* number of providers */
-	time_t		 	 expires; /* NotAfter of the ASPA EE cert */
+	time_t			 expires; /* NotAfter of the ASPA EE cert */
 };
 
 /*
@@ -474,6 +520,7 @@ struct stats {
 	size_t	 rrdp_fails; /* failed rrdp repositories */
 	size_t	 crls; /* revocation lists */
 	size_t	 gbrs; /* ghostbuster records */
+	size_t	 taks; /* signed TAL objects */
 	size_t	 aspas; /* ASPA objects */
 	size_t	 aspas_fail; /* ASPA objects failing syntactic parse */
 	size_t	 aspas_invalid; /* ASPAs with invalid customerASID */
@@ -540,10 +587,19 @@ void		 gbr_free(struct gbr *);
 struct gbr	*gbr_parse(X509 **, const char *, const unsigned char *,
 		    size_t);
 
+void		 geofeed_free(struct geofeed *);
+struct geofeed	*geofeed_parse(X509 **, const char *, char *, size_t);
+
 void		 rsc_free(struct rsc *);
 struct rsc	*rsc_parse(X509 **, const char *, const unsigned char *,
 		    size_t);
 
+void		 takey_free(struct takey *);
+void		 tak_free(struct tak *);
+struct tak	*tak_parse(X509 **, const char *, const unsigned char *,
+		    size_t);
+struct tak	*tak_read(struct ibuf *);
+
 void		 aspa_buffer(struct ibuf *, const struct aspa *);
 void		 aspa_free(struct aspa *);
 void		 aspa_insert_vaps(struct vap_tree *, struct aspa *, size_t *,
@@ -573,15 +629,19 @@ int		 valid_filename(const char *, size_t);
 int		 valid_uri(const char *, size_t, const char *);
 int		 valid_origin(const char *, const char *);
 int		 valid_x509(char *, X509_STORE_CTX *, X509 *, struct auth *,
-		    struct crl *, int);
+		    struct crl *, const char **);
 int		 valid_rsc(const char *, struct cert *, struct rsc *);
 int		 valid_econtent_version(const char *, const ASN1_INTEGER *);
 int		 valid_aspa(const char *, struct cert *, struct aspa *);
+int		 valid_geofeed(const char *, struct cert *, struct geofeed *);
 
 /* Working with CMS. */
 unsigned char	*cms_parse_validate(X509 **, const char *,
 		    const unsigned char *, size_t,
 		    const ASN1_OBJECT *, size_t *);
+int		 cms_parse_validate_detached(X509 **, const char *,
+		    const unsigned char *, size_t,
+		    const ASN1_OBJECT *, BIO *);
 
 /* Work with RFC 3779 IP addresses, prefixes, ranges. */
 
@@ -702,6 +762,7 @@ struct ibuf	*io_buf_recvfd(int, struct ibuf **);
 void		 x509_init_oid(void);
 int		 x509_get_aia(X509 *, const char *, char **);
 int		 x509_get_aki(X509 *, const char *, char **);
+int		 x509_get_sia(X509 *, const char *, char **);
 int		 x509_get_ski(X509 *, const char *, char **);
 int		 x509_get_expire(X509 *, const char *, time_t *);
 int		 x509_get_crl(X509 *, const char *, char **);
@@ -713,7 +774,7 @@ char		*x509_convert_seqnum(const char *, const ASN1_INTEGER *);
 int		 x509_location(const char *, const char *, const char *,
 		    GENERAL_NAME *, char **);
 int		 x509_inherits(X509 *);
-int	 	 x509_any_inherits(X509 *);
+int		 x509_any_inherits(X509 *);
 
 /* printers */
 char		*time2str(time_t);
@@ -726,6 +787,8 @@ void		 roa_print(const X509 *, const struct roa *);
 void		 gbr_print(const X509 *, const struct gbr *);
 void		 rsc_print(const X509 *, const struct rsc *);
 void		 aspa_print(const X509 *, const struct aspa *);
+void		 tak_print(const X509 *, const struct tak *);
+void		 geofeed_print(const X509 *, const struct geofeed *);
 
 /* Output! */
 
