@@ -1,4 +1,4 @@
-/*	$OpenBSD: http.c,v 1.68 2022/09/09 08:11:06 claudio Exp $ */
+/*	$OpenBSD: http.c,v 1.73 2022/11/02 16:50:51 claudio Exp $ */
 /*
  * Copyright (c) 2020 Nils Fisher <nils_fisher@hotmail.com>
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
@@ -211,6 +211,39 @@ http_info(const char *uri)
 	return buf;
 }
 
+/*
+ * Return IP address in presentation format.
+ */
+static const char *
+ip_info(const struct http_connection *conn)
+{
+	static char	ipbuf[NI_MAXHOST];
+
+	if (conn->res == NULL)
+		return "unknown";
+
+	if (getnameinfo(conn->res->ai_addr, conn->res->ai_addrlen, ipbuf,
+	    sizeof(ipbuf), NULL, 0, NI_NUMERICHOST) != 0)
+		return "unknown";
+
+	return ipbuf;
+}
+
+static const char *
+conn_info(const struct http_connection *conn)
+{
+	static char	 buf[100 + NI_MAXHOST];
+	const char	*uri;
+
+	if (conn->req == NULL)
+		uri = conn->host;
+	else
+		uri = conn->req->uri;
+
+	snprintf(buf, sizeof(buf), "%s (%s)", http_info(uri), ip_info(conn));
+	return buf;
+}
+
 /*
  * Determine whether the character needs encoding, per RFC2396.
  */
@@ -410,7 +443,8 @@ proxy_parse_uri(char *uri)
 
 	if ((proxy.proxyhost = strdup(host)) == NULL)
 		err(1, NULL);
-	proxy.proxyport = port;
+	if ((proxy.proxyport = strdup(port)) == NULL)
+		err(1, NULL);
 	proxy.proxyauth = cookie;
 
 	free(fullhost);
@@ -792,8 +826,7 @@ http_do(struct http_connection *conn, enum res (*f)(struct http_connection *))
 		conn->events = POLLOUT;
 		break;
 	default:
-		errx(1, "%s: unexpected function return",
-		    http_info(conn->host));
+		errx(1, "%s: unexpected function return", conn_info(conn));
 	}
 }
 
@@ -803,10 +836,6 @@ http_do(struct http_connection *conn, enum res (*f)(struct http_connection *))
 static enum res
 http_connect_done(struct http_connection *conn)
 {
-	freeaddrinfo(conn->res0);
-	conn->res0 = NULL;
-	conn->res = NULL;
-
 	if (proxy.proxyhost != NULL)
 		return proxy_connect(conn);
 	return http_tls_connect(conn);
@@ -819,6 +848,7 @@ static enum res
 http_connect(struct http_connection *conn)
 {
 	const char *cause = NULL;
+	struct addrinfo *res;
 
 	assert(conn->fd == -1);
 	conn->state = STATE_CONNECT;
@@ -829,9 +859,9 @@ http_connect(struct http_connection *conn)
 	else
 		conn->res = conn->res->ai_next;
 	for (; conn->res != NULL; conn->res = conn->res->ai_next) {
-		struct addrinfo *res = conn->res;
 		int fd, save_errno;
 
+		res = conn->res;
 		fd = socket(res->ai_family,
 		    res->ai_socktype | SOCK_NONBLOCK, res->ai_protocol);
 		if (fd == -1) {
@@ -870,8 +900,10 @@ http_connect(struct http_connection *conn)
 	}
 
 	if (conn->fd == -1) {
-		if (cause != NULL)
-			warn("%s: %s", http_info(conn->req->uri), cause);
+		if (cause != NULL) {
+			conn->res = res;
+			warn("%s: %s", conn_info(conn), cause);
+		}
 		return http_failed(conn);
 	}
 
@@ -889,22 +921,16 @@ http_finish_connect(struct http_connection *conn)
 
 	len = sizeof(error);
 	if (getsockopt(conn->fd, SOL_SOCKET, SO_ERROR, &error, &len) == -1) {
-		warn("%s: getsockopt SO_ERROR", http_info(conn->req->uri));
-		goto fail;
+		warn("%s: getsockopt SO_ERROR", conn_info(conn));
+		return http_connect_failed(conn);
 	}
 	if (error != 0) {
 		errno = error;
-		warn("%s: connect", http_info(conn->req->uri));
-		goto fail;
+		warn("%s: connect", conn_info(conn));
+		return http_connect_failed(conn);
 	}
 
 	return http_connect_done(conn);
-
-fail:
-	close(conn->fd);
-	conn->fd = -1;
-
-	return http_connect(conn);
 }
 
 /*
@@ -921,12 +947,12 @@ http_tls_connect(struct http_connection *conn)
 		return http_failed(conn);
 	}
 	if (tls_configure(conn->tls, tls_config) == -1) {
-		warnx("%s: TLS configuration: %s\n", http_info(conn->req->uri),
+		warnx("%s: TLS configuration: %s\n", conn_info(conn),
 		    tls_error(conn->tls));
 		return http_failed(conn);
 	}
 	if (tls_connect_socket(conn->tls, conn->fd, conn->host) == -1) {
-		warnx("%s: TLS connect: %s\n", http_info(conn->req->uri),
+		warnx("%s: TLS connect: %s\n", conn_info(conn),
 		    tls_error(conn->tls));
 		return http_failed(conn);
 	}
@@ -942,7 +968,7 @@ http_tls_handshake(struct http_connection *conn)
 {
 	switch (tls_handshake(conn->tls)) {
 	case -1:
-		warnx("%s: TLS handshake: %s", http_info(conn->req->uri),
+		warnx("%s: TLS handshake: %s", conn_info(conn),
 		    tls_error(conn->tls));
 		return http_failed(conn);
 	case TLS_WANT_POLLIN:
@@ -1074,7 +1100,7 @@ http_parse_status(struct http_connection *conn, char *buf)
 
 	cp = strchr(buf, ' ');
 	if (cp == NULL) {
-		warnx("Improper response from %s", http_info(conn->host));
+		warnx("Improper response from %s", conn_info(conn));
 		return -1;
 	} else
 		cp++;
@@ -1083,7 +1109,7 @@ http_parse_status(struct http_connection *conn, char *buf)
 	status = strtonum(ststr, 100, 599, &errstr);
 	if (errstr != NULL) {
 		strnvis(gerror, cp, sizeof gerror, VIS_SAFE);
-		warnx("Error retrieving %s: %s", http_info(conn->host),
+		warnx("Error retrieving %s: %s", conn_info(conn),
 		    gerror);
 		return -1;
 	}
@@ -1096,7 +1122,7 @@ http_parse_status(struct http_connection *conn, char *buf)
 	case 308:	/* Redirect: permanent redirect */
 		if (conn->req->redirect_loop++ > 10) {
 			warnx("%s: Too many redirections requested",
-			    http_info(conn->host));
+			    conn_info(conn));
 			return -1;
 		}
 		/* FALLTHROUGH */
@@ -1110,7 +1136,7 @@ http_parse_status(struct http_connection *conn, char *buf)
 		break;
 	default:
 		strnvis(gerror, cp, sizeof gerror, VIS_SAFE);
-		warnx("Error retrieving %s: %s", http_info(conn->host),
+		warnx("Error retrieving %s: %s", conn_info(conn),
 		    gerror);
 		return -1;
 	}
@@ -1187,7 +1213,7 @@ http_parse_header(struct http_connection *conn, char *buf)
 		conn->iosz = strtonum(cp, 0, MAX_CONTENTLEN, &errstr);
 		if (errstr != NULL) {
 			warnx("Content-Length of %s is %s",
-			    http_info(conn->req->uri), errstr);
+			    conn_info(conn), errstr);
 			return -1;
 		}
 	} else if (http_isredirect(conn) &&
@@ -1329,7 +1355,7 @@ read_more:
 	s = tls_read(conn->tls, conn->buf + conn->bufpos,
 	    conn->bufsz - conn->bufpos);
 	if (s == -1) {
-		warnx("%s: TLS read: %s", http_info(conn->host),
+		warnx("%s: TLS read: %s", conn_info(conn),
 		    tls_error(conn->tls));
 		return http_failed(conn);
 	} else if (s == TLS_WANT_POLLIN) {
@@ -1341,7 +1367,7 @@ read_more:
 	if (s == 0) {
 		if (conn->req)
 			warnx("%s: short read, connection closed",
-			    http_info(conn->req->uri));
+			    conn_info(conn));
 		return http_failed(conn);
 	}
 
@@ -1431,7 +1457,7 @@ again:
 		    conn->iosz > (off_t)conn->bufpos)
 			goto read_more;
 
-		/* got a full buffer full of data */
+		/* got a buffer full of data */
 		if (conn->req == NULL) {
 			/*
 			 * After redirects all data needs to be discarded.
@@ -1459,7 +1485,7 @@ again:
 		if (buf == NULL)
 			goto read_more;
 		if (http_parse_chunked(conn, buf) != 0) {
-			warnx("%s: bad chunk encoding", http_info(conn->host));
+			warnx("%s: bad chunk encoding", conn_info(conn));
 			free(buf);
 			return http_failed(conn);
 		}
@@ -1480,7 +1506,7 @@ again:
 			goto read_more;
 		/* expect empty line to finish a chunk of data */
 		if (*buf != '\0') {
-			warnx("%s: bad chunk encoding", http_info(conn->host));
+			warnx("%s: bad chunk encoding", conn_info(conn));
 			free(buf);
 			return http_failed(conn);
 		}
@@ -1518,7 +1544,7 @@ http_write(struct http_connection *conn)
 		s = tls_write(conn->tls, conn->buf + conn->bufpos,
 		    conn->bufsz - conn->bufpos);
 		if (s == -1) {
-			warnx("%s: TLS write: %s", http_info(conn->host),
+			warnx("%s: TLS write: %s", conn_info(conn),
 			    tls_error(conn->tls));
 			return http_failed(conn);
 		} else if (s == TLS_WANT_POLLIN) {
@@ -1553,14 +1579,14 @@ proxy_read(struct http_connection *conn)
 	s = read(conn->fd, conn->buf + conn->bufpos,
 	    conn->bufsz - conn->bufpos);
 	if (s == -1) {
-		warn("%s: read", http_info(conn->host));
+		warn("%s: read", conn_info(conn));
 		return http_failed(conn);
 	}
 
 	if (s == 0) {
 		if (conn->req)
 			warnx("%s: short read, connection closed",
-			    http_info(conn->host));
+			    conn_info(conn));
 		return http_failed(conn);
 	}
 
@@ -1614,7 +1640,7 @@ proxy_write(struct http_connection *conn)
 	s = write(conn->fd, conn->buf + conn->bufpos,
 	    conn->bufsz - conn->bufpos);
 	if (s == -1) {
-		warn("%s: write", http_info(conn->host));
+		warn("%s: write", conn_info(conn));
 		return http_failed(conn);
 	}
 	conn->bufpos += s;
@@ -1677,13 +1703,13 @@ data_write(struct http_connection *conn)
 
 	s = write(conn->req->outfd, conn->buf, bsz);
 	if (s == -1) {
-		warn("%s: data write", http_info(conn->req->uri));
+		warn("%s: data write", conn_info(conn));
 		return http_failed(conn);
 	}
 
 	conn->totalsz += s;
 	if (conn->totalsz > MAX_CONTENTLEN) {
-		warn("%s: too much data offered", http_info(conn->req->uri));
+		warn("%s: too much data offered", conn_info(conn));
 		return http_failed(conn);
 	}
 
@@ -1934,11 +1960,11 @@ proc_http(char *bind_addr, int fd)
 			else if (conn->io_time <= now) {
 				if (conn->state == STATE_CONNECT) {
 					warnx("%s: connect timeout",
-					    http_info(conn->host));
+					    conn_info(conn));
 					http_do(conn, http_connect_failed);
 				} else {
 					warnx("%s: timeout, connection closed",
-					    http_info(conn->host));
+					    conn_info(conn));
 					http_do(conn, http_failed);
 				}
 			}
