@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.219 2022/09/03 09:22:25 claudio Exp $ */
+/*	$OpenBSD: main.c,v 1.227 2022/11/30 08:16:10 job Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -25,7 +25,6 @@
 #include <sys/wait.h>
 
 #include <assert.h>
-#include <ctype.h>
 #include <err.h>
 #include <errno.h>
 #include <dirent.h>
@@ -33,6 +32,7 @@
 #include <fnmatch.h>
 #include <poll.h>
 #include <pwd.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -64,14 +64,22 @@ const char	*bird_tablename = "ROAS";
 int	verbose;
 int	noop;
 int	filemode;
+int	shortlistmode;
 int	rrdpon = 1;
 int	repo_timeout;
 time_t	deadline;
 
-struct skiplist skiplist = LIST_HEAD_INITIALIZER(skiplist);
-
 struct stats	 stats;
 
+struct fqdnlistentry {
+	LIST_ENTRY(fqdnlistentry)	 entry;
+	char				*fqdn;
+};
+LIST_HEAD(fqdns, fqdnlistentry);
+
+struct fqdns shortlist = LIST_HEAD_INITIALIZER(fqdns);
+struct fqdns skiplist = LIST_HEAD_INITIALIZER(fqdns);
+
 /*
  * Log a message to stderr if and only if "verbose" is non-zero.
  * This uses the err(3) functionality.
@@ -447,22 +455,35 @@ static void
 queue_add_from_cert(const struct cert *cert)
 {
 	struct repo		*repo;
-	struct skiplistentry	*sle;
+	struct fqdnlistentry	*le;
 	char			*nfile, *npath, *host;
 	const char		*uri, *repouri, *file;
 	size_t			 repourisz;
+	int			 shortlisted = 0;
 
-	LIST_FOREACH(sle, &skiplist, entry) {
-		if (strncmp(cert->repo, "rsync://", 8) != 0)
-			errx(1, "unexpected protocol");
-		host = cert->repo + 8;
+	if (strncmp(cert->repo, "rsync://", 8) != 0)
+		errx(1, "unexpected protocol");
+	host = cert->repo + 8;
 
-		if (strncasecmp(host, sle->value, strcspn(host, "/")) == 0) {
+	LIST_FOREACH(le, &skiplist, entry) {
+		if (strncasecmp(host, le->fqdn, strcspn(host, "/")) == 0) {
 			warnx("skipping %s (listed in skiplist)", cert->repo);
 			return;
 		}
 	}
 
+	LIST_FOREACH(le, &shortlist, entry) {
+		if (strncasecmp(host, le->fqdn, strcspn(host, "/")) == 0) {
+			shortlisted = 1;
+			break;
+		}
+	}
+	if (shortlistmode && shortlisted == 0) {
+		if (verbose)
+			warnx("skipping %s (not shortlisted)", cert->repo);
+		return;
+	}
+
 	repo = repo_lookup(cert->talid, cert->repo,
 	    rrdpon ? cert->notify : NULL);
 	if (repo == NULL)
@@ -595,8 +616,6 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	case RTYPE_GBR:
 		st->gbrs++;
 		break;
-	case RTYPE_FILE:
-		break;
 	case RTYPE_ASPA:
 		st->aspas++;
 		io_read_buf(b, &c, sizeof(c));
@@ -612,6 +631,11 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 			st->aspas_invalid++;
 		aspa_free(aspa);
 		break;
+	case RTYPE_TAK:
+		st->taks++;
+		break;
+	case RTYPE_FILE:
+		break;
 	default:
 		errx(1, "unknown entity type %d", type);
 	}
@@ -715,7 +739,7 @@ tal_load_default(void)
 static void
 load_skiplist(const char *slf)
 {
-	struct skiplistentry	*sle;
+	struct fqdnlistentry	*le;
 	FILE			*fp;
 	char			*line = NULL;
 	size_t			 linesize = 0, linelen;
@@ -744,12 +768,12 @@ load_skiplist(const char *slf)
 		if (!valid_uri(line, linelen, NULL))
 			errx(1, "invalid entry in skiplist: %s", line);
 
-		if ((sle = malloc(sizeof(struct skiplistentry))) == NULL)
+		if ((le = malloc(sizeof(struct fqdnlistentry))) == NULL)
 			err(1, NULL);
-		if ((sle->value = strdup(line)) == NULL)
+		if ((le->fqdn = strdup(line)) == NULL)
 			err(1, NULL);
 
-		LIST_INSERT_HEAD(&skiplist, sle, entry);
+		LIST_INSERT_HEAD(&skiplist, le, entry);
 		stats.skiplistentries++;
 	}
 
@@ -757,6 +781,26 @@ load_skiplist(const char *slf)
 	free(line);
 }
 
+/*
+ * Load shortlist entries.
+ */
+static void
+load_shortlist(const char *fqdn)
+{
+	struct fqdnlistentry	*le;
+
+	if (!valid_uri(fqdn, strlen(fqdn), NULL))
+		errx(1, "invalid fqdn passed to -q: %s", fqdn);
+
+	if ((le = malloc(sizeof(struct fqdnlistentry))) == NULL)
+		err(1, NULL);
+
+	if ((le->fqdn = strdup(fqdn)) == NULL)
+		err(1, NULL);
+
+	LIST_INSERT_HEAD(&shortlist, le, entry);
+}
+
 static void
 check_fs_size(int fd, const char *cachedir)
 {
@@ -862,7 +906,7 @@ main(int argc, char *argv[])
 	    "proc exec unveil", NULL) == -1)
 		err(1, "pledge");
 
-	while ((c = getopt(argc, argv, "b:Bcd:e:fjnorRs:S:t:T:vV")) != -1)
+	while ((c = getopt(argc, argv, "b:Bcd:e:fH:jnorRs:S:t:T:vV")) != -1)
 		switch (c) {
 		case 'b':
 			bind_addr = optarg;
@@ -883,6 +927,10 @@ main(int argc, char *argv[])
 			filemode = 1;
 			noop = 1;
 			break;
+		case 'H':
+			shortlistmode = 1;
+			load_shortlist(optarg);
+			break;
 		case 'j':
 			outformats |= FORMAT_JSON;
 			break;
@@ -895,7 +943,7 @@ main(int argc, char *argv[])
 		case 'R':
 			rrdpon = 0;
 			break;
-		case 'r':
+		case 'r': /* Remove after OpenBSD 7.3 */
 			rrdpon = 1;
 			break;
 		case 's':
@@ -1313,6 +1361,7 @@ main(int argc, char *argv[])
 	    stats.mfts, stats.mfts_fail, stats.mfts_stale);
 	printf("Certificate revocation lists: %zu\n", stats.crls);
 	printf("Ghostbuster records: %zu\n", stats.gbrs);
+	printf("Trust Anchor Keys: %zu\n", stats.taks);
 	printf("Repositories: %zu\n", stats.repos);
 	printf("Cleanup: removed %zu files, %zu directories, %zu superfluous\n",
 	    stats.del_files, stats.del_dirs, stats.extra_files);
@@ -1328,8 +1377,9 @@ usage:
 	fprintf(stderr,
 	    "usage: rpki-client [-BcjnoRrVv] [-b sourceaddr] [-d cachedir]"
 	    " [-e rsync_prog]\n"
-	    "                   [-S skiplist] [-s timeout] [-T table] [-t tal]"
-	    " [outputdir]\n"
+	    "                   [-H fqdn] [-S skiplist] [-s timeout] [-T table]"
+	    " [-t tal]\n"
+	    "                   [outputdir]\n"
 	    "       rpki-client [-Vv] [-d cachedir] [-j] [-t tal] -f file ..."
 	    "\n");
 	return 1;
