@@ -1,4 +1,4 @@
-/*	$OpenBSD: output.c,v 1.27 2022/08/30 18:56:49 job Exp $ */
+/*	$OpenBSD: output.c,v 1.28 2022/11/04 13:01:19 tb Exp $ */
 /*
  * Copyright (c) 2019 Theo de Raadt <deraadt@openbsd.org>
  *
@@ -73,7 +73,7 @@ static const struct outputs {
 	{ FORMAT_BIRD, "bird", output_bird2 },
 	{ FORMAT_CSV, "csv", output_csv },
 	{ FORMAT_JSON, "json", output_json },
-	{ 0, NULL }
+	{ 0, NULL, NULL }
 };
 
 static FILE	*output_createtmp(char *);
