@@ -1,4 +1,4 @@
-/*	$OpenBSD: repo.c,v 1.34 2022/05/15 16:43:34 tb Exp $ */
+/*	$OpenBSD: repo.c,v 1.39 2022/09/02 21:56:45 claudio Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -41,6 +41,8 @@ extern struct stats	stats;
 extern int		noop;
 extern int		rrdpon;
 extern int		repo_timeout;
+extern time_t		deadline;
+int			nofetch;
 
 enum repo_state {
 	REPO_LOADING = 0,
@@ -288,7 +290,7 @@ repo_done(const void *vp, int ok)
 		if (vp == rp->rsync)
 			entityq_flush(&rp->queue, rp);
 		if (vp == rp->rrdp) {
-			if (!ok) {
+			if (!ok && !nofetch) {
 				/* try to fall back to rsync */
 				rp->rrdp = NULL;
 				rp->rsync = rsync_get(rp->repouri,
@@ -383,7 +385,7 @@ ta_get(struct tal *tal)
 		err(1, NULL);
 	tr->basedir = repo_dir(tal->descr, "ta", 0);
 
-	/* steal URI infromation from TAL */
+	/* steal URI information from TAL */
 	tr->urisz = tal->urisz;
 	tr->uri = tal->uri;
 	tal->urisz = 0;
@@ -937,8 +939,8 @@ rrdp_finish(unsigned int id, int ok)
 		stats.rrdp_repos++;
 		rr->state = REPO_DONE;
 	} else {
-		warnx("%s: load from network failed, fallback to rsync",
-		    rr->notifyuri);
+		warnx("%s: load from network failed, fallback to %s",
+		    rr->notifyuri, nofetch ? "cache" : "rsync");
 		stats.rrdp_fails++;
 		rr->state = REPO_FAILED;
 		/* clear the RRDP repo since it failed */
@@ -1016,10 +1018,6 @@ ta_lookup(int id, struct tal *tal)
 	if ((rp->repouri = strdup(tal->descr)) == NULL)
 		err(1, NULL);
 
-	/* try to create base directory */
-	if (mkpath(rp->basedir) == -1)
-		warn("mkpath %s", rp->basedir);
-
 	/* check if sync disabled ... */
 	if (noop) {
 		logx("ta/%s: using cache", rp->repouri);
@@ -1027,6 +1025,10 @@ ta_lookup(int id, struct tal *tal)
 		return rp;
 	}
 
+	/* try to create base directory */
+	if (mkpath(rp->basedir) == -1)
+		warn("mkpath %s", rp->basedir);
+
 	rp->ta = ta_get(tal);
 
 	/* need to check if it was already loaded */
@@ -1044,7 +1046,6 @@ repo_lookup(int talid, const char *uri, const char *notify)
 {
 	struct repo	*rp;
 	char		*repouri;
-	int		 nofetch = 0;
 
 	if ((repouri = rsync_base_uri(uri)) == NULL)
 		errx(1, "bad caRepository URI: %s", uri);
@@ -1078,10 +1079,6 @@ repo_lookup(int talid, const char *uri, const char *notify)
 		nofetch = 1;
 	}
 
-	/* try to create base directory */
-	if (mkpath(rp->basedir) == -1)
-		warn("mkpath %s", rp->basedir);
-
 	/* check if sync disabled ... */
 	if (noop || nofetch) {
 		logx("%s: using cache", rp->basedir);
@@ -1089,6 +1086,10 @@ repo_lookup(int talid, const char *uri, const char *notify)
 		return rp;
 	}
 
+	/* try to create base directory */
+	if (mkpath(rp->basedir) == -1)
+		warn("mkpath %s", rp->basedir);
+
 	/* ... else try RRDP first if available then rsync */
 	if (notify != NULL)
 		rp->rrdp = rrdp_get(notify);
@@ -1204,22 +1205,54 @@ repo_fail(struct repo *rp)
 		errx(1, "%s: bad repo", rp->repouri);
 }
 
+static void
+repo_abort(struct repo *rp)
+{
+	/* reset the alarm */
+	rp->alarm = getmonotime() + repo_timeout;
+
+	if (rp->rsync)
+		rsync_abort(rp->rsync->id);
+	else if (rp->rrdp)
+		rrdp_abort(rp->rrdp->id);
+	else
+		repo_fail(rp);
+}
+
 int
 repo_check_timeout(int timeout)
 {
 	struct repo	*rp;
 	time_t		 now;
+	int		 diff;
 
 	now = getmonotime();
+
+	/* check against our runtime deadline first */
+	if (deadline != 0) {
+		if (deadline <= now) {
+			warnx("deadline reached, giving up on repository sync");
+			nofetch = 1;
+			/* clear deadline since nofetch is set */
+			deadline = 0;
+			/* increase now enough so that all pending repos fail */
+			now += repo_timeout;
+		} else {
+			diff = deadline - now;
+			diff *= 1000;
+			if (timeout == INFTIM || diff < timeout)
+				timeout = diff;
+		}
+	}
 	/* Look up in repository table. (Lookup should actually fail here) */
 	SLIST_FOREACH(rp, &repos, entry) {
 		if (repo_state(rp) == REPO_LOADING) {
 			if (rp->alarm <= now) {
 				warnx("%s: synchronisation timeout",
 				    rp->repouri);
-				repo_fail(rp);
+				repo_abort(rp);
 			} else {
-				int diff = rp->alarm - now;
+				diff = rp->alarm - now;
 				diff *= 1000;
 				if (timeout == INFTIM || diff < timeout)
 					timeout = diff;
@@ -1396,12 +1429,24 @@ repo_cleanup(struct filepath_tree *tree, int cachefd)
 			}
 			rr = repo_is_rrdp(e->fts_parent->fts_pointer);
 			if (rr != NULL) {
+				struct stat st;
 				char *fn;
 
 				if (asprintf(&fn, "%s/%s", rr->basedir,
 				    path) == -1)
 					err(1, NULL);
 
+				/*
+				 * If the file exists in the rrdp dir
+				 * that file is newer and needs to be kept
+				 * so unlink this file instead of moving
+				 * it over the file in the rrdp dir.
+				 */
+				if (fstatat(cachefd, fn, &st, 0) == 0 &&
+				    S_ISREG(st.st_mode)) {
+					free(fn);
+					goto unlink;
+				}
 				if (repo_mkpath(cachefd, fn) == 0) {
 					if (renameat(AT_FDCWD, e->fts_accpath,
 					    cachefd, fn) == -1)
@@ -1413,6 +1458,7 @@ repo_cleanup(struct filepath_tree *tree, int cachefd)
 				}
 				free(fn);
 			} else {
+ unlink:
 				if (unlink(e->fts_accpath) == -1) {
 					warn("unlink %s", path);
 				} else {
