@@ -1,4 +1,4 @@
-/*	$OpenBSD: rsync.c,v 1.43 2022/09/02 17:39:51 claudio Exp $ */
+/*	$OpenBSD: rsync.c,v 1.45 2022/11/29 20:26:22 job Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -19,7 +19,6 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <netinet/in.h>
-#include <assert.h>
 #include <err.h>
 #include <errno.h>
 #include <poll.h>
@@ -158,6 +157,7 @@ exec_rsync(const char *prog, const char *bind_addr, char *uri, char *dst,
 		args[i++] = "--include=*.mft";
 		args[i++] = "--include=*.roa";
 		args[i++] = "--include=*.asa";
+		args[i++] = "--include=*.tak";
 		args[i++] = "--exclude=*";
 		if (bind_addr != NULL) {
 			args[i++] = "--address";
