@@ -1,4 +1,4 @@
-/*	$OpenBSD: validate.c,v 1.45 2022/09/03 14:41:47 job Exp $ */
+/*	$OpenBSD: validate.c,v 1.51 2022/11/30 08:17:21 job Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -15,15 +15,12 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-#include <sys/socket.h>
-
 #include <arpa/inet.h>
 #include <assert.h>
 #include <ctype.h>
 #include <err.h>
 #include <fcntl.h>
 #include <inttypes.h>
-#include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -290,6 +287,8 @@ valid_uri(const char *uri, size_t usz, const char *proto)
 
 	if (proto != NULL) {
 		s = strlen(proto);
+		if (s >= usz)
+			return 0;
 		if (strncasecmp(uri, proto, s) != 0)
 			return 0;
 	}
@@ -365,20 +364,22 @@ build_crls(const struct crl *crl, STACK_OF(X509_CRL) **crls)
 }
 
 /*
- * Validate the X509 certificate.  If crl is NULL don't check CRL.
- * Returns 1 for valid certificates, returns 0 if there is a verify error
+ * Validate the X509 certificate. Returns 1 for valid certificates,
+ * returns 0 if there is a verify error and sets *errstr to the error
+ * returned by X509_verify_cert_error_string().
  */
 int
 valid_x509(char *file, X509_STORE_CTX *store_ctx, X509 *x509, struct auth *a,
-    struct crl *crl, int nowarn)
+    struct crl *crl, const char **errstr)
 {
 	X509_VERIFY_PARAM	*params;
 	ASN1_OBJECT		*cp_oid;
 	STACK_OF(X509)		*chain;
 	STACK_OF(X509_CRL)	*crls = NULL;
 	unsigned long		 flags;
-	int			 c;
+	int			 error;
 
+	*errstr = NULL;
 	build_chain(a, &chain);
 	build_crls(crl, &crls);
 
@@ -403,9 +404,8 @@ valid_x509(char *file, X509_STORE_CTX *store_ctx, X509 *x509, struct auth *a,
 	X509_STORE_CTX_set0_crls(store_ctx, crls);
 
 	if (X509_verify_cert(store_ctx) <= 0) {
-		c = X509_STORE_CTX_get_error(store_ctx);
-		if (!nowarn || verbose > 1)
-			warnx("%s: %s", file, X509_verify_cert_error_string(c));
+		error = X509_STORE_CTX_get_error(store_ctx);
+		*errstr = X509_verify_cert_error_string(error);
 		X509_STORE_CTX_cleanup(store_ctx);
 		sk_X509_free(chain);
 		sk_X509_CRL_free(crls);
@@ -521,3 +521,28 @@ valid_aspa(const char *fn, struct cert *cert, struct aspa *aspa)
 
 	return 0;
 }
+
+/*
+ * Validate Geofeed prefixes: check that the prefixes are contained.
+ * Returns 1 if valid, 0 otherwise.
+ */
+int
+valid_geofeed(const char *fn, struct cert *cert, struct geofeed *g)
+{
+	size_t	 i;
+	char	 buf[64];
+
+	for (i = 0; i < g->geoipsz; i++) {
+		if (ip_addr_check_covered(g->geoips[i].ip->afi,
+		    g->geoips[i].ip->min, g->geoips[i].ip->max, cert->ips,
+		    cert->ipsz) > 0)
+			continue;
+
+		ip_addr_print(&g->geoips[i].ip->ip, g->geoips[i].ip->afi, buf,
+		    sizeof(buf));
+		warnx("%s: Geofeed: uncovered IP: %s", fn, buf);
+		return 0;
+	}
+
+	return 1;
+}
